## 🥞 Database operations :

So, we've learned how to create own Schema, now we are going to go through 
database operations .

Before we go, first we need to initialize our database.

```python
import os
from painlessdb import Schema, PainlessDB

schema = {
    "users": {
        'username': Schema.types.text(),
        'password': Schema.types.text(),
        'is_active': Schema.types.bool(default=True)
    },
    
    "donations": {
        'username': Schema.types.text(),
        'amount': Schema.types.int(),
        'made_when': Schema.types.datetime()
    },
    
    "user_count": Schema.types.int()
}

db_path = os.path.join(os.getcwd(), 'test.pldb')  # .pldb is db file extension
database = PainlessDB(db_path, schema_data=schema)
```

Now this will initialize and create our Database.

### 🍤 Database.Create :
To create new data use `PainlessDB.create()`
```python
db_path = os.path.join(os.getcwd(), 'test.pldb') 
database = PainlessDB(db_path, schema_data=schema)

database.create('users', fields=database.fields(username='Gumball', password='Deexnuts69'))
database.create('users', fields=database.fields(username='Darwin', password='oofifsh420'))
```
🎉 We've just created our new users instance, the first param in `create()` 
method is the `Group` name, this is the group where you want to create 
this content/instance at. in our case it's `user`. and for `fields` 
param define your fields inside `database.fields()`

### 🍰 Database.Get :
To get the data use `PainlessDB.get()`

Now there's a few tweak to get `Static` and `Group` data. Let's start with group

#### Database.Get | Group
```python
db_path = os.path.join(os.getcwd(), 'test.pldb') 
database = PainlessDB(db_path, schema_data=schema)

result = database.get('users')
print(result)
```
In first param use the `Group` name you want to get your data from. 
it's `users` in this case.

Now, let's run it.

`Output:`
```commandline
[<painlessdb.core.data.DataObject object at 0x0000014EFE0ABC70>, <painlessdb.core.data.DataObject object at 0x0000014EFE0ABC70>]
```

here we go, we got the entire list of `users` Group. But you might also want
specific instance, like searching with specific fields, now let's query 
with fields value.

```python
db_path = os.path.join(os.getcwd(), 'test.pldb') 
database = PainlessDB(db_path, schema_data=schema)

result = database.get('users', where=database.where(username="Gumball"))
print(result)
```

`Output: `
```commandline
[<painlessdb.core.data.DataObject object at 0x0000014EFE0ABC70>]
```
Here we go, we just got the instance we want, but it's in a list. 
If your search query is designed to give only one instance then you 
can use `multiple=False` to get rid of the list, `Note: if you use 
multiple=False when your result has multiple instances then it will 
return the first one on the list.`

```python
result = database.get('users', where=database.where(username="Gumball"), multiple=False)
print(result)
```

`Output: `
```commandline
DataObject(users)
```

Now you have the one you wanted to get. But we haven't seen how to 
get the field value from it.

```python
result = database.get('users', where=database.where(username="Gumball"), multiple=False)
print(result.username)
print(result['password'])
```

`Output: `
```commandline
Gumball
Deexnuts69
```

You can get the value by using field attributes like `result.username` or like a dict `result['username']`

#### Database.Get | Static
Getting static data is really easy, you don't even need to do query or something.

```python
db_path = os.path.join(os.getcwd(), 'test.pldb') 
database = PainlessDB(db_path, schema_data=schema)

result = database.get('user_count')
print(result)
print(result.value)
```

`Output: `
```commandline
DataObject(user_count)
0
```
All you need is just specify the Static model name. And to get the 
value of that object use `.value` attribute.

This is all you need to know about `Database.Get`

### 🥗 Database.Update
Let's see how we can use `update` functionality. Let's update some of 
our data. And there's also a few tweak in updating `Group` and `Static` 
data.

#### Database.Update | Group
There's 2 ways to update data. let's see the first one.
```python
db_path = os.path.join(os.getcwd(), 'test.pldb') 
database = PainlessDB(db_path, schema_data=schema)

database.update('users', where=database.where(username='Gumball'), 
                field=database.fields(username='Gumball Watterson'))
```

We've updated user users `username`. First we used `where` to get the 
specific instance we want to change, and then we used `fields` this is 
where you put the updated field value, make sure to specify your field 
inside `database.fields()` In this case we canged it to 
`username='Gumball Watterson'`.
Now this instance's value has been changed. 

The second way to do this is getting the instance object first.

```python
result = database.get('users', where=database.where(username="Gumball Watterson"), multiple=False)
result.password = 'newcrankinpass0'
result.update()
```
You can change its value with its attributes and then just use `update()`.
And your instance has been updated.


#### Database.Update | Static
It's also easy to update `Static` values. Let's see how to do it.

```python
database.update('user_count', value=1)
```
Use the `Static` models name at the first param and use `value=` param to 
update it' value. You can also update it like `result.value=1` and then 
`result.update()` either way works.

This is all about `Database.Update`

### 🧇 Database.Delete :
To delete an instance from your group use `.delete()`.
```python
result = database.get('users', where=database.where(username="Gumball Watterson"), multiple=False)
result.delete()
```

You can also delete it like this:

```python
database.delete('users', where=database.where(username="Gumball Watterson"))
```

This is all about `Database.Delete`

###

[<b> More About Data Types > </b>](4.MoreAboutDtypes.md)

[<b> < Building schema </b>](2.BuildingSchema.md)
